@@ -1,14 +1,15 @@
 package Module::Installed::Tiny;
 
-our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2021-05-19'; # DATE
-our $DIST = 'Module-Installed-Tiny'; # DIST
-our $VERSION = '0.005'; # VERSION
-
 use strict;
 use warnings;
 
 use Exporter qw(import);
+
+our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
+our $DATE = '2022-06-10'; # DATE
+our $DIST = 'Module-Installed-Tiny'; # DIST
+our $VERSION = '0.006'; # VERSION
+
 our @EXPORT_OK = qw(module_installed module_source);
 
 our $SEPARATOR;
@@ -23,7 +24,12 @@ BEGIN {
 }
 
 sub _module_source {
-    my $name_pm = shift;
+    my ($name_pm, $opts) = @_;
+
+    $opts //= {};
+    $opts->{die} = 1 unless defined $opts->{die};
+
+    my $name_prefix;
 
     for my $entry (@INC) {
         next unless defined $entry;
@@ -45,6 +51,11 @@ sub _module_source {
                     or die "Can't locate $name_pm: $path: $!";
                 local $/;
                 return wantarray ? (scalar <$fh>, $path) : scalar <$fh>;
+            } elsif ($opts->{find_prefix}) {
+                $path =~ s/\.pm\z//;
+                if (-d $path) {
+                    return wantarray ? (undef, $path) : \$path;
+                }
             }
         }
 
@@ -78,11 +89,15 @@ sub _module_source {
         }
     }
 
-    die "Can't locate $name_pm in \@INC (\@INC contains: ".join(" ", @INC).")";
+    if ($opts->{die}) {
+        die "Can't locate $name_pm in \@INC (\@INC contains: ".join(" ", @INC).")";
+    } else {
+        return;
+    }
 }
 
 sub module_source {
-    my $name = shift;
+    my ($name, $opts) = @_;
 
     # convert Foo::Bar -> Foo/Bar.pm
     my $name_pm;
@@ -92,11 +107,13 @@ sub module_source {
         $name_pm = $name;
     }
 
-    _module_source $name_pm;
+    _module_source($name_pm, $opts);
 }
 
 sub module_installed {
-    my $name = shift;
+    my ($name, $opts) = @_;
+
+    local $opts->{die} = 1;
 
     # convert Foo::Bar -> Foo/Bar.pm
     my $name_pm;
@@ -108,7 +125,7 @@ sub module_installed {
 
     return 1 if exists $INC{$name_pm};
 
-    if (eval { _module_source $name_pm; 1 }) {
+    if (eval { _module_source($name_pm, $opts); 1 }) {
         1;
     } else {
         0;
@@ -130,7 +147,7 @@ Module::Installed::Tiny - Check if a module is installed, with as little code as
 
 =head1 VERSION
 
-This document describes version 0.005 of Module::Installed::Tiny (from Perl distribution Module-Installed-Tiny), released on 2021-05-19.
+This document describes version 0.006 of Module::Installed::Tiny (from Perl distribution Module-Installed-Tiny), released on 2022-06-10.
 
 =head1 SYNOPSIS
 
@@ -169,7 +186,11 @@ This module does not require any other module except L<Exporter>.
 
 =head1 FUNCTIONS
 
-=head2 module_installed($name) => bool
+=head2 module_installed
+
+Usage:
+
+ module_installed($name [ , \%opts ]) => bool
 
 Check that module named C<$name> is available to load. This means that: either
 the module file exists on the filesystem and searchable in C<@INC> and the
@@ -180,7 +201,19 @@ Note that this does not guarantee that the module can eventually be loaded
 successfully, as there might be syntax or runtime errors in the module's source.
 To check for that, one would need to actually load the module using C<require>.
 
-=head2 module_source($name) => str | (str, source_name)
+Options:
+
+=over
+
+=item *
+
+=back
+
+=head2 module_source
+
+Usage:
+
+ module_source($name [ , \%opts ]) => str | (str, source_name)
 
 Return module's source code, without actually loading it. Die on failure (e.g.
 module named C<$name> not found in C<@INC>).
@@ -191,11 +224,36 @@ In list context:
 
 will return the list:
 
-(str, source_name)
+ (str, source_name)
 
 where C<str> is the module source code and C<source_name> is source information
 (file path, or the @INC ref entry when entry is a ref).
 
+Options:
+
+=over
+
+=item * die
+
+Bool. Default true. If set to false, won't die upon failure but instead will
+return undef (or empty list in list context).
+
+=item * find_prefix
+
+Bool. If set to true, when a module (e.g. C<Foo/Bar.pm>) is not found in the
+fileysstem but its directory is (C<Foo/Bar/>), then instead of dying or
+returning undef/empty list, the function will return:
+
+ \$path
+
+in scalar context, or:
+
+ (undef, \$path)
+
+in list context.
+
+=back
+
 =head1 FAQ
 
 =head2 How to get module source without dying? I want to just get undef if module source is not available.
@@ -216,14 +274,6 @@ Please visit the project's homepage at L<https://metacpan.org/release/Module-Ins
 
 Source repository is at L<https://github.com/perlancar/perl-Module-Installed-Tiny>.
 
-=head1 BUGS
-
-Please report any bugs or feature requests on the bugtracker website L<https://github.com/perlancar/perl-Module-Installed-Tiny/issues>
-
-When submitting a bug or request, please include a test-file or a
-patch to an existing test-file that illustrates the bug or desired
-feature.
-
 =head1 SEE ALSO
 
 L<Module::Load::Conditional> provides C<check_install> which also does what
@@ -239,11 +289,36 @@ hooks, nor do they actually check that the module file is readable.
 
 perlancar <perlancar@cpan.org>
 
+=head1 CONTRIBUTING
+
+
+To contribute, you can send patches by email/via RT, or send pull requests on
+GitHub.
+
+Most of the time, you don't need to build the distribution yourself. You can
+simply modify the code, then test via:
+
+ % prove -l
+
+If you want to build the distribution (e.g. to try to install it locally on your
+system), you can install L<Dist::Zilla>,
+L<Dist::Zilla::PluginBundle::Author::PERLANCAR>, and sometimes one or two other
+Dist::Zilla plugin and/or Pod::Weaver::Plugin. Any additional steps required
+beyond that are considered a bug and can be reported to me.
+
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2021, 2020, 2016 by perlancar@cpan.org.
+This software is copyright (c) 2022, 2021, 2020, 2016 by perlancar <perlancar@cpan.org>.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
+=head1 BUGS
+
+Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Module-Installed-Tiny>
+
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
+
 =cut
